@@ -8,11 +8,25 @@ from typing import Literal
 from uuid import UUID
 
 import aiohttp
+import jwt
 
+from .const import (
+    AUTH_LOGIN_URL,
+    AUTH_TOKEN_URL,
+    BOOTSTRAP_GRAPHQL_URL,
+    BOOTSTRAP_QUERY,
+    CARBON_FOOTPRINT_URL,
+    CARBON_INTENSITY_URL,
+    USAGE_DAILY_URL,
+    USAGE_HALF_HOURLY_URL,
+)
 from .exceptions import (
+    OVOEnergyAPIInvalidResponse,
     OVOEnergyAPINoCookies,
     OVOEnergyAPINotAuthorized,
+    OVOEnergyAPINotFound,
     OVOEnergyNoAccount,
+    OVOEnergyNoCustomer,
 )
 from .models import (
     OVOCost,
@@ -23,6 +37,7 @@ from .models import (
     OVOHalfHourUsage,
     OVOInterval,
     OVOMeterReadings,
+    OVORates,
 )
 from .models.accounts import Account, BootstrapAccounts, Supply, SupplyPointInfo
 from .models.carbon_intensity import OVOCarbonIntensity, OVOCarbonIntensityForecast
@@ -34,13 +49,6 @@ from .models.footprint import (
     OVOFootprintGas,
 )
 from .models.oauth import OAuth
-from .models.plan import (
-    OVOPlanElectricity,
-    OVOPlanGas,
-    OVOPlanRate,
-    OVOPlans,
-    OVOPlanUnitRate,
-)
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -57,42 +65,41 @@ class OVOEnergy:
         """Initilalize."""
         self._client_session = client_session
 
+        self._customer_id: UUID | None = None
         self._bootstrap_accounts: BootstrapAccounts | None = None
         self._cookies: SimpleCookie | None = None
         self._oauth: OAuth | None = None
         self._username: str | None = None
+        self._account_ids: list[int] | None = None
 
     @property
     def account_id(self) -> int | None:
         """Return account id."""
-        if (
-            self.custom_account_id is not None
-            and self.account_ids is not None
-            and self.custom_account_id not in set(self.account_ids)
+        if self.custom_account_id is None and (
+            self.account_ids is None or len(self.account_ids) == 0
         ):
-            raise OVOEnergyNoAccount("Custom account not found in accounts")
+            raise OVOEnergyNoAccount("No account id set")
 
         return (
             self.custom_account_id
-            if self.custom_account_id is not None
-            else self._bootstrap_accounts.selected_account_id
-            if self._bootstrap_accounts
+            if self.custom_account_id
+            else self.account_ids[0]
+            if self.account_ids and len(self.account_ids) > 0
             else None
         )
 
     @property
     def account_ids(self) -> list[int] | None:
         """Return account ids."""
-        return (
-            self._bootstrap_accounts.account_ids if self._bootstrap_accounts else None
-        )
+        return self._account_ids
 
     @property
     def customer_id(self) -> UUID | None:
         """Return customer id."""
-        return (
-            self._bootstrap_accounts.customer_id if self._bootstrap_accounts else None
-        )
+        if self._customer_id is None:
+            raise OVOEnergyNoCustomer("No customer id set")
+
+        return self._customer_id
 
     @property
     def oauth(self) -> OAuth | None:
@@ -135,13 +142,13 @@ class OVOEnergy:
             method,
             url,
             cookies=self._cookies if with_cookies else None,
-            headers={
-                "Authorization": f"Bearer {self.oauth.access_token}"
-                if self.oauth
+            headers=(
+                {
+                    "Authorization": f"Bearer {self.oauth.access_token}",
+                }
+                if with_authorization and self.oauth
                 else None
-            }
-            if with_authorization
-            else None,
+            ),
             **kwargs,
         )
         with contextlib.suppress(aiohttp.ClientResponseError):
@@ -150,6 +157,9 @@ class OVOEnergy:
         if with_authorization and response.status in [401, 403]:
             raise OVOEnergyAPINotAuthorized(f"Not authorized: {response.status}")
 
+        if response.status == 404:
+            raise OVOEnergyAPINotFound(f"Endpoint not found: {response.status}")
+
         return response
 
     async def authenticate(
@@ -159,7 +169,7 @@ class OVOEnergy:
     ) -> bool:
         """Authenticate."""
         response = await self._request(
-            "https://my.ovoenergy.com/api/v2/auth/login",
+            AUTH_LOGIN_URL,
             "POST",
             json={
                 "username": username,
@@ -188,7 +198,7 @@ class OVOEnergy:
     async def get_token(self) -> OAuth | Literal[False]:
         """Get token."""
         response = await self._request(
-            "https://my.ovoenergy.com/api/v2/auth/token",
+            AUTH_TOKEN_URL,
             "GET",
             with_authorization=False,
         )
@@ -205,56 +215,182 @@ class OVOEnergy:
             expires_at=datetime.now() + timedelta(minutes=json_response["expiresIn"]),
         )
 
+        # Read JWT token
+        decoded_token = jwt.decode(
+            self._oauth.access_token, options={"verify_signature": False}
+        )
+
+        # Set customer id from sub claim
+        self._customer_id = decoded_token.get("sub")
+
+        # Set fallback account ids from permissions array if it is set
+        ids: list[int] = []
+        for permission in decoded_token.get("permissions", []):
+            if "account::" not in permission:
+                continue
+            account_id = permission.split("account::")[1]
+            if not account_id.isdigit():
+                continue
+            ids.append(int(account_id))
+
+        # Set fallback account ids if they are set
+        if len(ids) > 0:
+            self._account_ids = ids
+        else:
+            self._account_ids = None
+
         return self._oauth
 
     async def bootstrap_accounts(self) -> BootstrapAccounts:
         """Bootstrap accounts."""
         response = await self._request(
-            "https://smartpaymapi.ovoenergy.com/first-login/api/bootstrap/v2/",
-            "GET",
+            BOOTSTRAP_GRAPHQL_URL,
+            "POST",
+            json={
+                "operationName": "Bootstrap",
+                "query": BOOTSTRAP_QUERY,
+                "variables": {
+                    "customerId": self.customer_id,
+                },
+            },
         )
         json_response = await response.json()
 
-        self._bootstrap_accounts = BootstrapAccounts(
-            account_ids=json_response["accountIds"],
-            customer_id=UUID(json_response["customerId"]),
-            selected_account_id=json_response["selectedAccountId"],
-            is_first_login=json_response.get("isFirstLogin", None),
-            accounts=[
-                Account(
-                    account_id=account.get("accountId", None),
-                    is_payg=account.get("isPayg", None),
-                    is_blocked=account.get("isBlocked", None),
-                    supplies=[
-                        Supply(
-                            mpxn=supply["mpxn"],
-                            fuel=supply["fuel"],
-                            is_onboarding=supply["isOnboarding"],
-                            start=datetime.fromisoformat(supply["start"])
-                            if supply["start"]
-                            else None,
-                            is_payg=supply["isPayg"],
-                            supply_point_info=SupplyPointInfo(
-                                meter_type=supply["supplyPointInfo"]["meterType"],
-                                meter_not_found=supply["supplyPointInfo"][
-                                    "meterNotFound"
-                                ],
-                                address=supply["supplyPointInfo"]["address"],
-                            )
-                            if supply["supplyPointInfo"]
-                            else None,
-                        )
-                        for supply in account["supplies"]
-                    ]
-                    if "supplies" in account
-                    else None
-                    if "supplies" in account
-                    else None,
-                )
-                for account in json_response["accounts"]
+        if "data" not in json_response:
+            raise OVOEnergyAPIInvalidResponse("Missing 'data' key in response")
+        if "customer_nextV1" not in json_response["data"]:
+            raise OVOEnergyAPIInvalidResponse(
+                "Missing 'data.customer_nextV1' key in response"
+            )
+        if "id" not in json_response["data"]["customer_nextV1"]:
+            raise OVOEnergyAPIInvalidResponse(
+                "Missing 'data.customer_nextV1.id' key in response"
+            )
+        if (
+            "customerAccountRelationships"
+            not in json_response["data"]["customer_nextV1"]
+        ):
+            raise OVOEnergyAPIInvalidResponse(
+                "Missing 'data.customer_nextV1.customerAccountRelationships' key in response"
+            )
+        if (
+            "edges"
+            not in json_response["data"]["customer_nextV1"][
+                "customerAccountRelationships"
             ]
-            if "accounts" in json_response
-            else None,
+        ):
+            raise OVOEnergyAPIInvalidResponse(
+                "Missing 'data.customer_nextV1.customerAccountRelationships.edges' key in response"
+            )
+
+        accounts: list[Account] = []
+        for edge in json_response["data"]["customer_nextV1"][
+            "customerAccountRelationships"
+        ]["edges"]:
+            if "node" not in edge:
+                _LOGGER.warning(
+                    "Missing 'data.customer_nextV1.customerAccountRelationships.edges[X].node' key in response"
+                )
+                continue
+            if "account" not in edge["node"]:
+                _LOGGER.warning(
+                    "Missing 'data.customer_nextV1.customerAccountRelationships.edges[X].node.account' key in response"
+                )
+                continue
+
+            if "id" not in edge["node"]["account"]:
+                _LOGGER.warning(
+                    "Missing 'data.customer_nextV1.customerAccountRelationships.edges[X].node.account.id' key in response"
+                )
+                continue
+
+            if "accountSupplyPoints" not in edge["node"]["account"]:
+                _LOGGER.warning(
+                    "Missing 'data.customer_nextV1.customerAccountRelationships.edges[X].node.account.accountSupplyPoints' key in response"
+                )
+                continue
+
+            supplies: list[Supply] = []
+            for supply in edge["node"]["account"]["accountSupplyPoints"]:
+                if "supplyPoint" not in supply:
+                    _LOGGER.warning(
+                        "Missing 'data.customer_nextV1.customerAccountRelationships.edges[X].node.account.accountSupplyPoints[X].supplyPoint' key in response"
+                    )
+                    continue
+
+                if "meterTechnicalDetails" not in supply["supplyPoint"]:
+                    _LOGGER.warning(
+                        "Missing 'data.customer_nextV1.customerAccountRelationships.edges[X].node.account.accountSupplyPoints[X].supplyPoint.meterTechnicalDetails' key in response"
+                    )
+                    continue
+
+                active_meter_technical_details = None
+                for meter_detail in supply["supplyPoint"]["meterTechnicalDetails"]:
+                    if "status" not in meter_detail:
+                        _LOGGER.warning(
+                            "Missing 'data.customer_nextV1.customerAccountRelationships.edges[X].node.account.accountSupplyPoints[X].supplyPoint.meterTechnicalDetails[X].status' key in response"
+                        )
+                        continue
+
+                    if meter_detail["status"].lower() == "active":
+                        active_meter_technical_details = meter_detail
+                        break
+
+                supply_point_address_lines: list[str] = []
+                if "address" not in supply["supplyPoint"]:
+                    _LOGGER.warning(
+                        "Missing 'data.customer_nextV1.customerAccountRelationships.edges[X].node.account.accountSupplyPoints[X].supplyPoint.address' key in response. Allowing empty address."
+                    )
+                else:
+                    supply_point_address_lines = supply["supplyPoint"]["address"].get(
+                        "addressLines", []
+                    )
+                    if "postCode" in supply["supplyPoint"]["address"]:
+                        supply_point_address_lines.append(
+                            supply["supplyPoint"]["address"].get("postCode")
+                        )
+
+                supplies.append(
+                    Supply(
+                        mpxn=active_meter_technical_details["meterSerialNumber"]
+                        if active_meter_technical_details
+                        else None,
+                        fuel=supply["supplyPoint"].get("fuelType", None),
+                        is_onboarding=supply["supplyPoint"].get("isOnboarding", None),
+                        start=supply["supplyPoint"].get("startDate", None),
+                        is_payg=supply["supplyPoint"].get("isPayg", None),
+                        supply_point_info=SupplyPointInfo(
+                            meter_type=active_meter_technical_details["type"]
+                            if active_meter_technical_details
+                            else None,
+                            meter_not_found=active_meter_technical_details[
+                                "status"
+                            ].lower()
+                            == "removed"
+                            if active_meter_technical_details
+                            else None,
+                            address=supply_point_address_lines,
+                        ),
+                    )
+                )
+
+            accounts.append(
+                Account(
+                    account_id=edge["node"]["account"]["id"],
+                    is_payg=None,  # No longer supplied
+                    is_blocked=None,  # No longer supplied
+                    supplies=supplies,
+                )
+            )
+
+        self._bootstrap_accounts = BootstrapAccounts(
+            account_ids=[account.account_id for account in accounts],
+            customer_id=json_response["data"]["customer_nextV1"]["id"],
+            selected_account_id=accounts[
+                0
+            ].account_id,  # We no longer get this, so pick the first one, the user should specify otherwise
+            is_first_login=False,  # We no longer get this, so assume false
+            accounts=accounts,
         )
 
         return self._bootstrap_accounts
@@ -264,16 +400,13 @@ class OVOEnergy:
         date: str,
     ) -> OVODailyUsage:
         """Get daily usage data."""
-        if self.account_id is None:
-            raise OVOEnergyNoAccount("No account found")
-
         ovo_usage = OVODailyUsage(
             electricity=None,
             gas=None,
         )
 
         response = await self._request(
-            f"https://smartpaymapi.ovoenergy.com/usage/api/daily/{self.account_id}?date={date}",
+            f"{USAGE_DAILY_URL}/{self.account_id}?date={date}",
             "GET",
         )
         json_response = await response.json()
@@ -287,28 +420,40 @@ class OVOEnergy:
                         ovo_usage.electricity.append(
                             OVODailyElectricity(
                                 consumption=usage.get("consumption", None),
-                                interval=OVOInterval(
-                                    start=datetime.fromisoformat(
-                                        usage["interval"]["start"]
-                                    ),
-                                    end=datetime.fromisoformat(
-                                        usage["interval"]["end"]
-                                    ),
-                                )
-                                if "interval" in usage
-                                else None,
-                                meter_readings=OVOMeterReadings(
-                                    start=usage["meterReadings"]["start"],
-                                    end=usage["meterReadings"]["end"],
-                                )
-                                if "meterReadings" in usage
-                                else None,
+                                interval=(
+                                    OVOInterval(
+                                        start=datetime.fromisoformat(
+                                            usage["interval"]["start"]
+                                        ),
+                                        end=datetime.fromisoformat(
+                                            usage["interval"]["end"]
+                                        ),
+                                    )
+                                    if "interval" in usage
+                                    else None
+                                ),
+                                meter_readings=(
+                                    OVOMeterReadings(
+                                        start=usage["meterReadings"]["start"],
+                                        end=usage["meterReadings"]["end"],
+                                    )
+                                    if "meterReadings" in usage
+                                    else None
+                                ),
                                 has_half_hour_data=usage.get("hasHalfHourData", None),
-                                cost=OVOCost(
-                                    amount=usage["cost"]["amount"],
-                                    currency_unit=usage["cost"]["currencyUnit"],
+                                cost=(
+                                    OVOCost(
+                                        amount=usage["cost"]["amount"],
+                                        currency_unit=usage["cost"]["currencyUnit"],
+                                    )
+                                    if "cost" in usage
+                                    else None
+                                ),
+                                rates=OVORates(
+                                    anytime=usage["rates"].get("anytime", None),
+                                    standing=usage["rates"].get("standing", None),
                                 )
-                                if "cost" in usage
+                                if "rates" in usage
                                 else None,
                             )
                         )
@@ -323,27 +468,39 @@ class OVOEnergy:
                             OVODailyGas(
                                 consumption=usage.get("consumption", None),
                                 volume=usage.get("volume", None),
-                                interval=OVOInterval(
-                                    start=datetime.fromisoformat(
-                                        usage["interval"]["start"]
-                                    ),
-                                    end=datetime.fromisoformat(
-                                        usage["interval"]["end"]
-                                    ),
-                                )
-                                if "interval" in usage
-                                else None,
-                                meter_readings=OVOMeterReadings(
-                                    start=usage["meterReadings"]["start"],
-                                    end=usage["meterReadings"]["end"],
-                                )
-                                if "meterReadings" in usage
-                                else None,
+                                interval=(
+                                    OVOInterval(
+                                        start=datetime.fromisoformat(
+                                            usage["interval"]["start"]
+                                        ),
+                                        end=datetime.fromisoformat(
+                                            usage["interval"]["end"]
+                                        ),
+                                    )
+                                    if "interval" in usage
+                                    else None
+                                ),
+                                meter_readings=(
+                                    OVOMeterReadings(
+                                        start=usage["meterReadings"]["start"],
+                                        end=usage["meterReadings"]["end"],
+                                    )
+                                    if "meterReadings" in usage
+                                    else None
+                                ),
                                 has_half_hour_data=usage.get("hasHalfHourData", None),
                                 cost=OVOCost(
                                     amount=usage["cost"]["amount"],
                                     currency_unit=usage["cost"]["currencyUnit"],
-                                ),
+                                )
+                                if "cost" in usage
+                                else None,
+                                rates=OVORates(
+                                    anytime=usage["rates"].get("anytime", None),
+                                    standing=usage["rates"].get("standing", None),
+                                )
+                                if "rates" in usage
+                                else None,
                             )
                         )
 
@@ -354,16 +511,13 @@ class OVOEnergy:
         date: str,
     ) -> OVOHalfHourUsage:
         """Get half hourly usage data."""
-        if self.account_id is None:
-            raise OVOEnergyNoAccount("No account found")
-
         ovo_usage = OVOHalfHourUsage(
             electricity=None,
             gas=None,
         )
 
         response = await self._request(
-            f"https://smartpaymapi.ovoenergy.com/usage/api/half-hourly/{self.account_id}?date={date}",
+            f"{USAGE_HALF_HOURLY_URL}/{self.account_id}?date={date}",
             "GET",
         )
         json_response = await response.json()
@@ -411,105 +565,10 @@ class OVOEnergy:
 
         return ovo_usage
 
-    async def get_plans(self) -> OVOPlans:
-        """Get plans."""
-        if self.account_id is None:
-            raise OVOEnergyNoAccount("No account found")
-
-        response = await self._request(
-            f"https://smartpaymapi.ovoenergy.com/orex/api/plans/{self.account_id}",
-            "GET",
-        )
-        json_response = await response.json()
-
-        return OVOPlans(
-            electricity=[
-                OVOPlanElectricity(
-                    name=json_response["electricity"]["name"],
-                    exit_fee=OVOPlanRate(
-                        amount=json_response["electricity"]["exitFee"]["amount"],
-                        currency_unit=json_response["electricity"]["exitFee"][
-                            "currencyUnit"
-                        ],
-                    ),
-                    contract_start_date=json_response["electricity"][
-                        "contractStartDate"
-                    ],
-                    contract_end_date=json_response["electricity"]["contractEndDate"],
-                    contract_type=json_response["electricity"]["contractType"],
-                    is_in_renewal=json_response["electricity"]["isInRenewal"],
-                    has_future_contracts=json_response["electricity"][
-                        "hasFutureContracts"
-                    ],
-                    mpxn=json_response["electricity"]["mpxn"],
-                    msn=json_response["electricity"]["msn"],
-                    personal_projection=json_response["electricity"][
-                        "personalProjection"
-                    ],
-                    standing_charge=OVOPlanRate(
-                        amount=json_response["electricity"]["standingCharge"]["amount"],
-                        currency_unit=json_response["electricity"]["standingCharge"][
-                            "currencyUnit"
-                        ],
-                    ),
-                    unit_rates=[
-                        OVOPlanUnitRate(
-                            name=unit_rate["name"],
-                            unit_rate=OVOPlanRate(
-                                amount=unit_rate["unitRate"]["amount"],
-                                currency_unit=unit_rate["unitRate"]["currencyUnit"],
-                            ),
-                        )
-                        for unit_rate in json_response["electricity"]["unitRates"]
-                    ],
-                )
-                for json_response in json_response["electricity"]
-            ],
-            gas=[
-                OVOPlanGas(
-                    name=json_response["gas"]["name"],
-                    exit_fee=OVOPlanRate(
-                        amount=json_response["gas"]["exitFee"]["amount"],
-                        currency_unit=json_response["gas"]["exitFee"]["currencyUnit"],
-                    ),
-                    contract_start_date=json_response["gas"]["contractStartDate"],
-                    contract_end_date=json_response["gas"]["contractEndDate"],
-                    contract_type=json_response["gas"]["contractType"],
-                    is_in_renewal=json_response["gas"]["isInRenewal"],
-                    has_future_contracts=json_response["gas"]["hasFutureContracts"],
-                    mpxn=json_response["gas"]["mpxn"],
-                    msn=json_response["gas"]["msn"],
-                    personal_projection=json_response["gas"]["personalProjection"],
-                    standing_charge=OVOPlanRate(
-                        amount=json_response["gas"]["standingCharge"]["amount"],
-                        currency_unit=json_response["gas"]["standingCharge"][
-                            "currencyUnit"
-                        ],
-                    ),
-                    unit_rates=[
-                        OVOPlanUnitRate(
-                            name=unit_rate["name"],
-                            unit_rate=OVOPlanRate(
-                                amount=unit_rate["unitRate"]["amount"],
-                                currency_unit=unit_rate["unitRate"]["currencyUnit"],
-                            ),
-                        )
-                        for unit_rate in json_response["gas"]["unitRates"]
-                    ],
-                )
-                for json_response in json_response["gas"]
-            ]
-            if "gas" in json_response
-            else None,
-        )
-
     async def get_footprint(self) -> OVOFootprint:
         """Get footprint."""
-        if self.account_id is None:
-            raise OVOEnergyNoAccount("No account found")
-
         response = await self._request(
-            f"https://smartpaymapi.ovoenergy.com/carbon-api/{self.account_id}/footprint",
+            f"{CARBON_FOOTPRINT_URL}/{self.account_id}/footprint",
             "GET",
         )
         json_response = await response.json()
@@ -552,7 +611,7 @@ class OVOEnergy:
     async def get_carbon_intensity(self):
         """Get carbon intensity."""
         response = await self._request(
-            "https://smartpaymapi.ovoenergy.com/carbon-bff/carbonintensity",
+            CARBON_INTENSITY_URL,
             "GET",
         )
         json_response = await response.json()
