@@ -15,9 +15,21 @@ class OVOEnergyAPIException(OVOEnergyException):
     """Exception for API exceptions."""
 
 
+class OVOEnergyAPIInvalidResponse(OVOEnergyAPIException):
+    """Exception for invalid response."""
+
+
 class OVOEnergyAPINotAuthorized(OVOEnergyAPIException):
     """Exception for API client not authorized."""
 
 
+class OVOEnergyAPINotFound(OVOEnergyAPIException):
+    """Exception for API endpoint not found (404)."""
+
+
 class OVOEnergyAPINoCookies(OVOEnergyAPIException):
     """Exception for no cookies found."""
+
+
+class OVOEnergyNoCustomer(OVOEnergyException):
+    """Exception for no customer found."""
